/*****************************************************************************
**
**  ctl::json version 2.110
**
**  Copyright (c) 2019-2024, Nozomu Katoo. All rights reserved.
**
**  Redistribution and use in source and binary forms, with or without
**  modification, are permitted provided that the following conditions are
**  met:
**
**  1. Redistributions of source code must retain the above copyright notice,
**     this list of conditions and the following disclaimer.
**
**  2. Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in the
**     documentation and/or other materials provided with the distribution.
**
**  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
**  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
**  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
**  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
**  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
**  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
**  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
**  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
**  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
**  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
**  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
**/

#ifndef CTL_JSON_TEMPLATE_LIBRARY
#define CTL_JSON_TEMPLATE_LIBRARY

#include <cstring>
#include <limits>
#include <algorithm>
#include <utility>
#include <cstdlib>
#include <string>
#include <vector>
#include <map>
#include <memory>

#if !defined(NAMESPACE_CTLJSON)
#define NAMESPACE_CTLJSON ctl
#endif

namespace NAMESPACE_CTLJSON
{
	namespace string_internal_
	{

template <typename T>
struct casetables
{
	static const T lc[];
	static const T uc[];
};
template <typename T>
const T casetables<T>::lc[] =
{
	0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39,	//  "0123456789";
	0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6a,	//  "abcdefghij"
	0x6b, 0x6c, 0x6d, 0x6e, 0x6f, 0x70, 0x71, 0x72, 0x73, 0x74,	//  "klmnopqrst"
	0x75, 0x76, 0x77, 0x78, 0x79, 0x7a, 0x2e, 0x2b,	//  "uvwxyz.+"
	0x2d, 0x69, 0x6e, 0x66,	//  "-inf"
	0x2d, 0x71, 0x6e, 0x61, 0x6e,	//  "-qnan"
	0x2d, 0x73, 0x6e, 0x61, 0x6e	//  "-snan"
};
template <typename T>
const T casetables<T>::uc[] =
{
	0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39,	//  "0123456789";
	0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4a,	//  "ABCDEFGHIJ"
	0x4b, 0x4c, 0x4d, 0x4e, 0x4f, 0x50, 0x51, 0x52, 0x53, 0x54,	//  "KLMNOPQRST"
	0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x2e, 0x2b,	//  "UVWXYZ.+"
	0x2d, 0x49, 0x4e, 0x46,	//  "-INF"
	0x2d, 0x51, 0x4e, 0x41, 0x4e,	//  "-QNAN"
	0x2d, 0x53, 0x4e, 0x41, 0x4e	//  "-SNAN"
};

template <typename stringT>
stringT reverse_(stringT &s)
{
	typedef typename stringT::size_type size_type;
	const size_type mid = s.size() / 2;

	for (size_type i = 0; i < mid; ++i)
		std::swap(s[i], s[s.size() - i - 1]);

	return s;
}

template <typename stringT>
int mul2_(stringT &s, const int radix)
{
	typedef typename stringT::value_type char_type;
	typedef typename stringT::size_type size_type;
	char_type carried = 0;

	for (size_type i = 0; i < s.size(); ++i)
	{
		char_type &ch = s[i] *= 2;

		carried = ((ch = static_cast<char_type>(ch + carried)) >= static_cast<char_type>(radix)) ? (ch = static_cast<char_type>(ch - radix), 1) : 0;
	}
	return carried ? (s.push_back(carried), 1) : 0;
}

template <typename stringT>
int div2_(stringT &s, const int radix)
{
	typedef typename stringT::value_type char_type;
	typedef typename stringT::size_type size_type;
	int borrowed = 0;

	for (size_type i = s.size(); i;)
	{
		char_type &ch = s[--i];

		ch = static_cast<char_type>(ch + (borrowed ? radix : 0));

		borrowed = ch & 1;
		ch /= 2;
	}
	if (borrowed)
		s.insert(0, 1, static_cast<char_type>(radix / 2));

	if (s.size() && s[s.size() - 1] == 0)
	{
		s.resize(s.size() - 1);
		borrowed |= 2;
	}
	return borrowed;
}

template <typename stringT>
int plus_(stringT &s1, const stringT &s2, const int radix)
{
	typedef typename stringT::value_type char_type;
	typedef typename stringT::size_type size_type;
	char_type carried = 0;

	if (s1.size() < s2.size())
		s1.resize(s2.size(), 0);

	for (size_type i = 0; i < s1.size(); ++i)
	{
		char_type &ch = s1[i] += (i < s2.size() ? s2[i] : 0) + carried;
		carried = (ch >= static_cast<char_type>(radix)) ? (ch = static_cast<char_type>(ch - radix), 1) : 0;
	}
	return carried ? (s1.push_back(carried), 1) : 0;
}

template <typename stringT>
int round_(stringT &s, typename stringT::size_type pos, const int radix)
{
	typedef typename stringT::value_type char_type;

	if (pos <= s.size())
	{
		char_type carried = (s[(pos = s.size() - pos)] >= static_cast<char_type>(radix / 2)) ? 1 : 0;

		for (s.erase(0, pos + 1), pos = 0; pos < s.size(); ++pos)
		{
			char_type &ch = s[pos];

			ch = static_cast<char_type>(ch + carried);
			carried = (ch >= static_cast<char_type>(radix)) ? (ch = static_cast<char_type>(ch - radix), 1) : 0;
		}
		if (carried)
			return s.push_back(carried), 1;
	}
	return 0;
}

template <typename T>
static bool is_same_(const T l, const T r)
{
	return std::memcmp(reinterpret_cast<const unsigned char *>(&l), reinterpret_cast<const unsigned char *>(&r), sizeof (T)) == 0;
}

	}	//  namespace string_internal_

template <typename stringT, typename T>
inline stringT to_string(T value, int base = 10, const int precision = 1)
{
	typedef typename stringT::value_type char_type;
	typedef typename stringT::size_type size_type;
	const char_type *const table = (base >= 0 ? string_internal_::casetables<char_type>::lc : (base = 0 - base, string_internal_::casetables<char_type>::uc));
	stringT num;

	if (base >= 2 && base <= 36)
	{
		const bool minus = value < 0 ? (value = 0 - value, true) : false;

		for (; value; value /= base)
		{
			num.push_back(table[value % base]);
		}

		if (precision > 0 && num.size() < static_cast<size_type>(precision))
			num.append(static_cast<size_type>(precision) - num.size(), table[0]);

		if (minus)
			num.push_back(table[38]);
	}
	return string_internal_::reverse_(num);
}

template <typename stringT, typename T>
stringT to_string_fp(T value, int fmt, int precision)
{
	typedef stringT string_type;
	typedef typename stringT::value_type char_type;
	typedef typename stringT::size_type size_type;

	const bool uc = fmt >= 'A' && fmt <= 'Z';

	const char_type *const table = uc ? (fmt += 'a' - 'A', string_internal_::casetables<char_type>::uc) : string_internal_::casetables<char_type>::lc;

	int dppos;

	if ((dppos = 38 + 1, string_internal_::is_same_(value, std::numeric_limits<T>::infinity())) || (--dppos, string_internal_::is_same_(value, -std::numeric_limits<T>::infinity())))
//		return &(uc ? "-INF" : "-inf")[dppos];
		return stringT(&table[dppos], &table[dppos + ((dppos & 1) ? 3 : 4)]);

	if ((dppos = 42 + 1, string_internal_::is_same_(value, std::numeric_limits<T>::quiet_NaN())) || (--dppos, string_internal_::is_same_(value, -std::numeric_limits<T>::quiet_NaN())))
//		return &(uc ? "-QNAN" : "-qnan")[dppos];
		return stringT(&table[dppos], &table[dppos + ((dppos & 1) ? 4 : 5)]);

	if ((dppos = 47 + 1, string_internal_::is_same_(value, std::numeric_limits<T>::signaling_NaN())) || (--dppos, string_internal_::is_same_(value, -std::numeric_limits<T>::signaling_NaN())))
//		return &(uc ? "-SNAN" : "-snan")[dppos];
		return stringT(&table[dppos], &table[dppos + ((dppos & 1) ? 5 : 4)]);

	const int base = fmt == 'a' ? (fmt = 'e', 16) : 10;
	const bool remove0s = (precision < 0) ? (precision = 0 - precision, true) : (fmt == 'g');
	const bool minus = (value < 0 || string_internal_::is_same_(value, static_cast<T>(-0.0))) ? (value = 0 - value, true) : false;
	string_type resstr;

	dppos = 0;
	if (value != 0.0)
	{
		int borrowed = 0;
		T mask = 1.0;
		string_type mstr(1, 1);

		if (value >= mask)
			for (; (value - mask) >= mask; mask *= 2.0)
				dppos += string_internal_::mul2_(mstr, base);
		else
			for (; value < ((dppos -= (string_internal_::div2_(mstr, base) & 2) ? 1 : 0), mask /= 2.0););

		for (int count = 11; value > 0.0 && mask > 0.0 && count < 64; mask /= 2.0)
		{
			if (borrowed)
				resstr.insert(0, 1, 0);

			if (value >= mask)
			{
				value -= mask;
				dppos += string_internal_::plus_(resstr, mstr, base);
			}
			borrowed = string_internal_::div2_(mstr, base) & 1;
		}

		if (base == 16)
		{
			const char_type &lb = resstr[resstr.size() - 1];
			const int shift = (lb & 8) ? 3 : ((lb & 4) ? 2 : ((lb & 2) ? 1 : 0));

			dppos = dppos * 4 + shift;
			if (shift)
			{
				char_type borrowed2 = 0;
				for (size_type i = resstr.size(); i;)
				{
					char_type &ch = resstr[--i] |= borrowed2;
					borrowed2 = ch << 4;
					ch = static_cast<char_type>((ch >> shift) & 15);
				}
				if (borrowed2)
					resstr.insert(0, 1, static_cast<char_type>((borrowed2 >> shift) & 15));
			}
		}
//		dppos += round_(resstr, 18, base);
	}

	if (fmt == 'g')
	{
		dppos += string_internal_::round_(resstr, precision + 1, base);

		if (dppos >= -4 && dppos < precision)
		{
			precision = precision - 1 - dppos;
			dppos < 0 ? (resstr.append(-dppos, 0), dppos = 1) : ++dppos;
			goto STYLE_F;
		}
		goto STYLE_E;
	}

	if (fmt == 'e')
	{
		++precision;
		dppos += string_internal_::round_(resstr, precision + 1, base);

		STYLE_E:

		if (remove0s)
			for (size_type i = 0; precision > 1 && i < resstr.size() && resstr[i] == 0; --precision, ++i);

		if (resstr.size() > static_cast<size_type>(precision))
			resstr.erase(0, resstr.size() - precision);
		else
			resstr.insert(0, static_cast<size_type>(precision) - resstr.size(), 0);

		for (size_type i = 0; i < resstr.size(); ++i)
			resstr[i] = table[static_cast<unsigned int>(resstr[i])];

		if (resstr.size() > 1)
			resstr.insert(resstr.size() - 1, 1, table[36]);

		resstr.insert(0, 1, table[base == 10 ? 14 : 25]);	//  'E''e' 'P''p'
		resstr.insert(0, 1, ((dppos >= 0) ? table[37] : table[38]));

		if (dppos < 0)
			dppos = 0 - dppos;

		const size_type minwidth = base == 10 ? 2 : 1;
		for (size_type inspos = 0; dppos || inspos < minwidth; ++inspos, dppos /= 10)
			resstr.insert(inspos, 1, table[dppos % 10]);

		if (base == 16)
		{
			resstr.push_back(table[33]);	//  'X' 'x'
			resstr.push_back(table[0]);
		}
	}
	else
	{
		dppos < 0 ? (resstr.append(-dppos, 0), dppos = 1) : ++dppos;

		dppos += string_internal_::round_(resstr, precision + dppos + 1, base);

		STYLE_F:

		if (remove0s)
			for (size_type i = 0; precision > 1 && i < resstr.size() && resstr[i] == 0; --precision, ++i);

		precision += dppos;

		if (resstr.size() > static_cast<size_type>(precision))
			resstr.erase(0, resstr.size() - precision);
		else
		{
			const size_type wantedsize = static_cast<size_type>(remove0s ? dppos : precision);
			if (resstr.size() < wantedsize)
				resstr.insert(0, wantedsize - resstr.size(), 0);
		}

		for (size_type i = 0; i < resstr.size(); ++i)
			resstr[i] = table[static_cast<unsigned int>(resstr[i])];

		if (resstr.size() > static_cast<size_type>(dppos))
			resstr.insert(resstr.size() - dppos, 1, table[36]);
	}

	if (minus)
		resstr.push_back(table[38]);

	return string_internal_::reverse_(resstr);
}

template <typename stringT>
inline stringT to_string(double val, const int fmt = 'f', const int precision = 6)
{
	return to_string_fp<stringT>(val, fmt, precision);
}
template <typename stringT>
inline stringT to_string(float val, const int fmt = 'f', const int precision = 6)
{
	return to_string_fp<stringT>(val, fmt, precision);
}
template <typename stringT>
inline stringT to_string(long double val, const int fmt = 'f', const int precision = 6)
{
	return to_string_fp<stringT>(val, fmt, precision);
}

	namespace json_internal_
	{

struct utf8tag {};
struct utf16tag {};

template <bool, typename T>
struct enable_if {};
template <typename T>
struct enable_if<true, T> { typedef T type; };
template <typename IntType, typename RetType>
struct enable_if_integer : enable_if<std::numeric_limits<IntType>::is_integer, RetType> {};

#if defined(__cplusplus) && (__cplusplus >= 201103L)
template <typename charT, typename utftype,
	template <typename V1, typename... V2> class vector = std::vector,
	template <typename M1, typename M2, typename... M3> class map = std::map,
	template <typename A1> class alloc = std::allocator>
#else
template <typename charT, typename utftype,
	template <typename V1, typename V2> class vector = std::vector,
	template <typename M1, typename M2, typename M3, typename T4> class map = std::map,
	template <typename A1> class alloc = std::allocator>
#endif
class jsonnode
{
public:

	enum value_type
	{
		array, object, number, string, boolean, null, unassigned, fallback
	};

	typedef std::basic_string<charT, std::char_traits<charT>, alloc<charT> > string_type;
	typedef charT char_type;

private:

	jsonnode(const value_type v) : /* value_(NULL), */ type_(v)	//  For the fallback node.
	{
	}

public:

	jsonnode() : /* value_(NULL), */ type_(unassigned)
	{
	}

	jsonnode(const jsonnode &right) : /* value_(NULL), */ type_(right.type_)
	{
		if (this != &right && !is_fallback() && !right.is_fallback())
		{
			if (type_ & parsing)
			{
				const int nmtype = type_ & ~parsing;

				if (nmtype == array)
					array_ = right.array_;
				else if (nmtype == object)
					object_ = right.object_;
				else if (nmtype == number || nmtype == string)
					string_ = right.string_;
				else if (nmtype == boolean)
					boolvalue_ = right.boolvalue_;
			}
			else if (type_ == array)
				array_ = new array_type(*right.array_);
			else if (type_ == object)
				object_ = new object_type(*right.object_);
			else if (type_ == number || type_ == string)
				string_ = new string_type(*right.string_);
			else if (type_ == boolean)
				boolvalue_ = right.boolvalue_;
		}
	}

	template <typename StringT>
	explicit jsonnode(const StringT &s) : /* value_(NULL), */ type_(unassigned)
	{
		parse(s);
	}

	template <typename ForwardIterator>
	jsonnode(ForwardIterator begin, const ForwardIterator end) : /* value_(NULL), */ type_(unassigned)
	{
		parse(begin, end);
	}

	~jsonnode()
	{
		if (type_ < boolean)
			destroy_node_();
	}

	void clear()
	{
		if (!is_fallback())
		{
			destroy_node_();
			type_ = unassigned;
		}
	}

	//  Types.

	value_type type() const
	{
		return static_cast<value_type>(type_);
	}

	bool is_num() const
	{
		return type_ == number;
	}
	bool is_str() const
	{
		return type_ == string;
	}
	bool is_bool() const
	{
		return type_ == boolean;
	}
	bool is_true() const
	{
		return type_ == boolean && boolvalue_;
	}
	bool is_false() const
	{
		return type_ == boolean && !boolvalue_;
	}
	bool is_null() const
	{
		return type_ == null;
	}
	bool is_array() const
	{
		return type_ == array;
	}
	bool is_object() const
	{
		return type_ == object;
	}
	bool is_unassigned() const
	{
		return type_ == unassigned;
	}
	bool is_fallback() const
	{
		return type_ == fallback;
	}
	bool is_assigned() const
	{
		return type_ < unassigned;
	}

	bool exists() const
	{
		return type_ != fallback;
	}

	jsonnode &operator=(const jsonnode &right)
	{
		if (this != &right && !is_fallback() && !right.is_fallback())
		{
			const int newtype = right.type_;

			if (newtype & parsing)
			{
				const int nmtype = newtype & ~parsing;

				if (type_ < boolean)	//  [[unlikely]]
					destroy_node_();
				if (nmtype == array)
					array_ = right.array_;
				else if (nmtype == object)
					object_ = right.object_;
				else if (nmtype == number || nmtype == string)
					string_ = right.string_;
				else if (nmtype == boolean)
					boolvalue_ = right.boolvalue_;
			}
			else if (newtype == array)
			{
				array_type *const newarray = new array_type(*right.array_);
				destroy_node_();
				array_ = newarray;
			}
			else if (newtype == object)
			{
				object_type *const newobject = new object_type(*right.object_);
				destroy_node_();
				object_ = newobject;
			}
			else if (newtype == number || newtype == string)
			{
				string_type *const newstring = new string_type(*right.string_);
				destroy_node_();
				string_ = newstring;
			}
			else if (newtype == boolean)
			{
				const bool bv = right.boolvalue_;
				destroy_node_();
				boolvalue_ = bv;
			}
			type_ = newtype;
		}
		return *this;
	}

	jsonnode &operator=(const double d)
	{
		return set_num(d, -6);
	}

	template <typename IntType>
	typename enable_if_integer<IntType, jsonnode>::type &operator=(const IntType i)
	{
		return set_num(static_cast<double>(i), -6);
	}

	jsonnode &operator=(const bool b)
	{
		return set_bool(b);
	}

	jsonnode &operator=(const string_type &s)
	{
		return set_str(s);
	}

	jsonnode &operator=(const char_type *const p)
	{
		return set_str(p);
	}

	template <typename StringT>
	bool parse(const StringT &s)
	{
		if (!is_fallback())
		{
			const typename StringT::value_type *const end = s.data() + s.size();
			return parse(s.c_str(), end) == end && type_ != unassigned;
		}
		return false;
	}

	template <typename charT2>
	bool parse(const charT2 *const p)
	{
		if (!is_fallback())
		{
			const charT2 *const end = p + std::char_traits<charT2>::length(p);
			return parse(p, end) == end && type_ != unassigned;
		}
		return false;
	}

	template <typename ForwardIterator>
	ForwardIterator parse(ForwardIterator begin, const ForwardIterator end)
	{
		if (is_fallback())
			return begin;

		destroy_node_();
		type_ = unassigned;
		begin = skip_spaces_(begin, end);

		parse_value_(begin, end);
		remove_parsingflag();

		return begin;
	}

	string_type to_string() const
	{
		string_type out;
		value_to_string_(out);
		return out;
	}
	string_type stringify() const
	{
		string_type out;
		value_to_string_(out);
		return out;
	}

	template <typename StringT>
	StringT to_string() const
	{
		StringT out;
		value_to_string_(out);
		return out;
	}
	template <typename StringT>
	StringT stringify() const
	{
		StringT out;
		value_to_string_(out);
		return out;
	}

	//  For number.

	jsonnode &set_num(const double d, const int precision)
	{
		if (is_fallback())
			return fallback_node_;

		if (type_ != number)
		{
			destroy_node_();
			type_ = number;
			string_ = new string_type();
		}

		const safe_double dbl(d);
		string_->assign(reinterpret_cast<const char_type *>(&dbl.dbl), double_size);
		string_->append(ctl::to_string<string_type>(d, 'f', precision));
		return *this;
	}

	//  For string.

	jsonnode &set_str(const string_type &s)
	{
		return set_str(s.data(), s.data() + s.size());
	}

	jsonnode &set_str(const char_type *const p)
	{
		return set_str(p, p + std::char_traits<char_type>::length(p));
	}

	template <typename InputIterator>
	jsonnode &set_str(InputIterator begin, const InputIterator end)
	{
		if (is_fallback())
			return fallback_node_;

		if (type_ != string)
		{
			destroy_node_();
			type_ = string;
			string_ = new string_type(begin, end);
		}
		else
			string_->assign(begin, end);

		return *this;
	}

	//  For array.

	bool exists(const std::size_t no) const
	{
		return !is_fallback() && n(no) != &fallback_node_;
	}

	const jsonnode &operator[](const std::size_t no) const
	{
		const jsonnode *const node = n(no);
		return *node;
	}
	jsonnode &operator[](const std::size_t no)
	{
		jsonnode *const node = n(no);
		return *node;
	}

	const jsonnode *n(const std::size_t no) const
	{
		if (type_ == array)
		{
			if (no < array_->size())
				return &(*array_)[no];
		}
		return &fallback_node_;
	}
	jsonnode *n(const std::size_t no)
	{
		if (type_ == array)
		{
			if (no < array_->size())
				return &(*array_)[no];
		}
		return &fallback_node_;
	}

	jsonnode &operator()(const std::size_t no)
	{
		if (is_fallback())
			return fallback_node_;

		if (type_ == array)
		{
			if (no >= array_->size())
				array_->resize(no + 1);
		}
		else
		{
			destroy_node_();
			type_ = array;
			array_ = new array_type();
			array_->resize(no + 1);
		}
		return (*array_)[no];
	}

	bool erase(const std::size_t no)
	{
		if (type_ == array && no < array_->size())
		{
			array_->erase(array_->begin() + no);
			return true;
		}
		return false;
	}

	bool insert(const std::size_t no, const jsonnode &right)
	{
		if (type_ == array && no <= array_->size())
		{
			array_->insert(array_->begin() + no, right);
			return true;
		}
		return false;
	}

	void push_back(const jsonnode &right)
	{
		if (type_ == array)
			array_->push_back(right);
	}

	//  For array and object.

	std::size_t size() const
	{
		if (type_ == array)
			return array_->size();
		else if (type_ == object)
			return object_->objmap.size();

		return 0;
	}

	//  For object.

	bool exists(const string_type &key) const
	{
		return !is_fallback() && n(key) != &fallback_node_;
	}

	const jsonnode &operator[](const string_type &key) const
	{
		const jsonnode *const node = n(key);
		return *node;
	}
	jsonnode &operator[](const string_type &key)
	{
		jsonnode *const node = n(key);
		return *node;
	}

	const jsonnode *n(const string_type &key) const
	{
		if (type_ == object)
		{
			const typename object_core::const_iterator it = object_->objmap.find(key);

			if (it != object_->objmap.end())
				return &it->second;
		}
		return &fallback_node_;
	}
	jsonnode *n(const string_type &key)
	{
		if (type_ == object)
		{
			const typename object_core::iterator it = object_->objmap.find(key);

			if (it != object_->objmap.end())
				return &it->second;
		}
		return &fallback_node_;
	}

	jsonnode &operator()(const string_type &key)
	{
		if (is_fallback())
			return fallback_node_;

		if (type_ == object)
		{
			const typename object_core::iterator it = object_->objmap.find(key);

			if (it != object_->objmap.end())
				return it->second;

			const safe_keysize keylen1(static_cast<keysize_type>(key.size()));
			object_->order.append(reinterpret_cast<const char_type *>(&keylen1.size), keysize_size);
			object_->order.append(key);
			return object_->objmap[key];
		}

		destroy_node_();
		type_ = object;
		object_ = new object_type();
		const safe_keysize keylen0(static_cast<keysize_type>(key.size()));
		object_->order.append(reinterpret_cast<const char_type *>(&keylen0.size), keysize_size);
		object_->order.append(key);
		return object_->objmap[key];
	}

	bool erase(const string_type &key)
	{
		if (type_ == object)
		{
			const typename object_core::iterator it = object_->objmap.find(key);

			if (it != object_->objmap.end())
			{
				object_->objmap.erase(it);
				remove_orderkey_(key);
				return true;
			}
		}
		return false;
	}

	bool insert(const string_type &inspos, const string_type &newkey, const jsonnode &newelem)
	{
		if (type_ == object && newkey != inspos)
		{
			strsize_type newkeyindex = object_->order.size();
			strsize_type insposindex = object_->order.size();

			for (strsize_type i = 0; (i < object_->order.size()) && (newkeyindex == object_->order.size() || insposindex == object_->order.size());)
			{
				const strsize_type keylen = static_cast<strsize_type>(*reinterpret_cast<const keysize_type *>(&object_->order[i]));
				strsize_type base = i + keysize_size;

				if (keylen == newkey.size() && object_->order.compare(base, keylen, newkey, 0, keylen) == 0)
					newkeyindex = i;
				else if (keylen == inspos.size() && object_->order.compare(base, keylen, inspos, 0, keylen) == 0)
					insposindex = i;

				i = base + keylen;
			}

			if (insposindex < object_->order.size())
			{
				const strsize_type newkeylen = newkey.size();

				if (newkeyindex < object_->order.size())
				{
					const strsize_type remlen = keysize_size + newkeylen;

					if ((newkeyindex + remlen) == insposindex)
						goto UPDATE;
					else
					{
						object_->order.erase(newkeyindex, remlen);
						if (newkeyindex < insposindex)
							insposindex -= remlen;
					}
				}
				{
					const safe_keysize nkl(newkeylen);

					object_->order.insert(insposindex, newkey);
					object_->order.insert(insposindex, reinterpret_cast<const char_type *>(&nkl.size), keysize_size);
				}

				UPDATE:
				object_->objmap[newkey] = newelem;
				return true;
			}
		}
		return false;
	}

	bool push_back(const string_type &key, const jsonnode &right)
	{
		if (type_ == object)
		{
			object_->objmap[key] = right;

			for (strsize_type pos = 0; (pos + keysize_size) < object_->order.size();)
			{
				const keysize_type len = *reinterpret_cast<const keysize_type *>(&object_->order[pos]);

				pos += keysize_size;
				if (static_cast<keysize_type>(key.size()) == len)
				{
					for (keysize_type i = 0;; ++i)
					{
						if (i == len)
						{
							if (pos + len == object_->order.size())
								return true;

							object_->order.erase(pos - keysize_size, keysize_size + len);
							goto PUSH_KEY;
						}
						if (object_->order[pos + i] != key[i])
							break;
					}
				}
				pos += len;
			}

			PUSH_KEY:
			const safe_keysize keylen0(static_cast<keysize_type>(key.size()));
			object_->order.append(reinterpret_cast<const char_type *>(&keylen0.size), keysize_size);
			object_->order.append(key);
			return true;
		}
		return false;
	}

	double num() const
	{
		if (type_ == number)
			return *reinterpret_cast<const double *>(string_->c_str());
		if (type_ == boolean)
			return static_cast<double>(boolvalue_);
		return 0.0;
	}

	template <typename NumType>
	NumType num() const
	{
		if (type_ == number)
			return static_cast<NumType>(*reinterpret_cast<const double *>(string_->c_str()));
		if (type_ == boolean)
			return static_cast<NumType>(boolvalue_);
		return static_cast<NumType>(0);
	}

	string_type numstr() const
	{
		static const char_type ltrue[] = { 0x74, 0x72, 0x75, 0x65, 0 };	//  "true"
		static const char_type lfalse[] = { 0x66, 0x61, 0x6C, 0x73, 0x65, 0 };	//  "false"
		static const char_type lnull[] = { 0x6E, 0x75, 0x6C, 0x6C, 0 };	//  "null"

		if (type_ == number)
			return string_->substr(double_size);
		if (type_ == boolean)
			return boolvalue_ ? ltrue : lfalse;
		if (type_ == null)
			return lnull;
		return string_type();
	}

	string_type str() const
	{
		if (type_ == string)
			return *string_;
		return string_type();
	}

	//  For boolean, null.

	jsonnode &set_bool(const bool b)
	{
		if (is_fallback())
			return fallback_node_;

		if (type_ != boolean)
		{
			destroy_node_();
			type_ = boolean;
		}
		boolvalue_ = b;
		return *this;
	}

	jsonnode &set_null()
	{
		if (is_fallback())
			return fallback_node_;

		if (type_ != null)
		{
			destroy_node_();
			type_ = null;
		}
		return *this;
	}

private:

	static const int parsing = 16;

	typedef typename string_type::size_type strsize_type;
	typedef unsigned int keysize_type;

	typedef vector<jsonnode, alloc<jsonnode> > array_type;
	typedef map<string_type, jsonnode, std::less<string_type>, alloc<std::pair<const string_type, jsonnode> > > object_core;

	struct object_type
	{
		object_core objmap;
		string_type order;
	};

	static const std::size_t keysize_size = sizeof (keysize_type) / sizeof (char_type) + ((sizeof (keysize_type) % sizeof (char_type)) ? 1 : 0);
	static const std::size_t double_size = sizeof (double) / sizeof (char_type) + ((sizeof (double) % sizeof (char_type)) ? 1 : 0);

	union safe_keysize
	{
		keysize_type size;
		char_type padding_[keysize_size];
		safe_keysize(const keysize_type s) : size(s) {}
	};

	union safe_double
	{
		double dbl;
		char_type padding_[double_size];
		safe_double(const double d) : dbl(d) {}
	};

	template <typename ForwardIterator>
	inline ForwardIterator skip_spaces_(ForwardIterator begin, const ForwardIterator end)
	{
		for (; begin != end && (*begin == 0x20 || *begin == 0x09 || *begin == 0x0a || *begin == 0x0d); ++begin);

		return begin;
	}

	//  UTF-8 -> UTF-8.

	template <typename ForwardIterator, typename UTFtag>
	bool parse_string_(string_type *const str, ForwardIterator &begin, const ForwardIterator end, const UTFtag)
	{
		ForwardIterator prevbegin = begin;
		unsigned long prev = 0ul;

		for (; begin != end;)
		{
			if (*begin == 0x22)	//  '"'
			{
				if (prevbegin != begin)
					str->append(prevbegin, begin);

				begin = skip_spaces_(++begin, end);
				return true;
			}

			if (*begin == 0x5C)	//  '\\'
			{
				if (++begin == end)
					break;

				char_type nch = *begin;

				switch (nch)
				{
				case 0x22:	//  "    quotation mark  U+0022
				case 0x5c:	//  \    reverse solidus U+005C
				case 0x2f:	//  /    solidus         U+002F
					break;

				case 0x62:	//  b    backspace       U+0008
					nch = 0x08;
					break;

				case 0x66:	//  f    form feed       U+000C
					nch = 0x0c;
					break;

				case 0x6e:	//  n    line feed       U+000A
					nch = 0x0a;
					break;

				case 0x72:	//  r    carriage return U+000D
					nch = 0x0d;
					break;

				case 0x74:	//  t    tab             U+0009
					nch = 0x09;
					break;

				case 0x75:	//  uXXXX                U+XXXX
					{
						ForwardIterator upos = begin;
						unsigned long ucp = parse_4hexdigits_(begin, end);

						if (ucp < 0x10000ul)
						{
							++begin;

							if (prev && ucp >= 0xDC00ul && ucp <= 0xDFFFul)
							{
								ucp = ((ucp & 0x3FFul) | ((prev & 0x3FFul) << 10)) + 0x10000;

								(*str)[str->size() - 3] = static_cast<char_type>(((ucp >> 18) & 7) | 0xf0);
								(*str)[str->size() - 2] = static_cast<char_type>(((ucp >> 12) & 0x3f) | 0x80);
								(*str)[str->size() - 1] = static_cast<char_type>(((ucp >> 6) & 0x3f) | 0x80);
								str->append(1, static_cast<char_type>((ucp & 0x3f) | 0x80));
								prev = 0ul;
							}
							else
							{
								prev = 0ul;

								if (ucp < 0x80ul)
								{
									str->append(prevbegin, upos);
									(*str)[str->size() - 1] = static_cast<char_type>(ucp);
								}
								else if (ucp < 0x800ul)
								{
									++upos;
									str->append(prevbegin, upos);
									(*str)[str->size() - 2] = static_cast<char_type>(((ucp >> 6) & 0x1f) | 0xc0);
									(*str)[str->size() - 1] = static_cast<char_type>((ucp & 0x3f) | 0x80);
								}
								else
								{
									++upos;
									++upos;
									str->append(prevbegin, upos);
									(*str)[str->size() - 3] = static_cast<char_type>(((ucp >> 12) & 0x0f) | 0xe0);
									(*str)[str->size() - 2] = static_cast<char_type>(((ucp >> 6) & 0x3f) | 0x80);
									(*str)[str->size() - 1] = static_cast<char_type>((ucp & 0x3f) | 0x80);
									if ((ucp & 0xFC00ul) == 0xD800ul)
										prev = ucp;
								}
							}
							prevbegin = begin;
							continue;
						}
					}
					//  "\u" not followed by four hexdigits.
					prev = 0ul;
					continue;

				default:
					prev = 0ul;
					++begin;
					continue;
				}

				str->append(prevbegin, begin);
				(*str)[str->size() - 1] = nch;

				prev = 0ul;
				prevbegin = ++begin;
				continue;
			}
			++begin;
		}
		str->clear();
		return false;
	}

	//  UTF-8 -> UTF-16.

	template <typename ForwardIterator>
	bool parse_string_(string_type *const str, ForwardIterator &begin, const ForwardIterator end, const utf16tag)
	{
		ForwardIterator prevbegin = begin;

		for (; begin != end;)
		{
			if (*begin == 0x22)	//  '"'
			{
				if (prevbegin != begin)
					str->append(prevbegin, begin);

				begin = skip_spaces_(++begin, end);
				return true;
			}

			if (*begin == 0x5C)	//  '\\'
			{
				if (++begin == end)
					break;

				char_type nch = *begin;

				switch (nch)
				{
				case 0x22:	//  "    quotation mark  U+0022
				case 0x5c:	//  \    reverse solidus U+005C
				case 0x2f:	//  /    solidus         U+002F
					break;

				case 0x62:	//  b    backspace       U+0008
					nch = 0x08;
					break;

				case 0x66:	//  f    form feed       U+000C
					nch = 0x0c;
					break;

				case 0x6e:	//  n    line feed       U+000A
					nch = 0x0a;
					break;

				case 0x72:	//  r    carriage return U+000D
					nch = 0x0d;
					break;

				case 0x74:	//  t    tab             U+0009
					nch = 0x09;
					break;

				case 0x75:	//  uXXXX                U+XXXX
					{
						ForwardIterator upos = begin;
						unsigned long ucp = parse_4hexdigits_(begin, end);

						if (ucp < 0x10000ul)
						{
							++begin;

							str->append(prevbegin, upos);
							(*str)[str->size() - 1] = static_cast<char_type>(ucp);
							prevbegin = begin;
						}
					}
					//  "\u" not followed by four hexdigits.
					continue;

				default:
					++begin;
					continue;
				}

				str->append(prevbegin, begin);
				(*str)[str->size() - 1] = nch;

				prevbegin = ++begin;
				continue;
			}
			else if (*begin & 0x80)
			{
				unsigned long ch = *begin & 0xff;

				if (ch >= 0xc2)
				{
					if ((ch & 0x20) == 0)	//  2 octets?
					{
						if (++begin != end && (*begin & 0xc0) == 0x80)	//  2nd octet.
						{
							ch = ((ch << 6) & 0x7c0) | (*begin & 0x3f);

							str->append(prevbegin, begin);
							(*str)[str->size() - 1] = static_cast<char_type>(ch);
							prevbegin = ++begin;
							continue;
						}
					}
					else if ((ch & 0x10) == 0)	//  3 octets?
					{
						if (++begin != end && (*begin & 0xc0) == 0x80)	//  2nd octet.
						{
							const ForwardIterator tmpend = begin;

							ch = (ch << 8) | (*begin & 0xff);

							if (ch >= 0xe0a0)	//  E0A080..
							{
								if (++begin != end && (*begin & 0xc0) == 0x80)	//  3rd octet.
								{
									ch = ((ch << 4) & 0xf000) | ((ch << 6) & 0xfc0) | (*begin & 0x3f);

									str->append(prevbegin, tmpend);
									(*str)[str->size() - 1] = static_cast<char_type>(ch);
									prevbegin = ++begin;
									continue;
								}
							}
						}
					}
					else if (ch >= 0xf0 && ch <= 0xf4)	//  4 octets?
					{
						if (++begin != end && (*begin & 0xc0) == 0x80)	//  2nd octet.
						{
							ch = (ch << 8) | (*begin & 0xff);

							if (ch >= 0xf090 && ch <= 0xf48f)	//  F0908080..F48FBFBF.
							{
								if (++begin != end && (*begin & 0xc0) == 0x80)	//  3rd octet.
								{
									const ForwardIterator tmpend = begin;

									ch = ((ch << 4) & 0x7000) | ((ch << 6) & 0xfc0) | (*begin & 0x3f);

									if (++begin != end && (*begin & 0xc0) == 0x80)	//  4th octet.
									{
										ch = ((ch << 6) | (*begin & 0x3f)) - 0x10000;

										str->append(prevbegin, tmpend);
										(*str)[str->size() - 2] = static_cast<char_type>(0xd800 | (ch >> 10));
										(*str)[str->size() - 1] = static_cast<char_type>(0xdc00 | (ch & 0x3ff));
										prevbegin = ++begin;
										continue;
									}
								}
							}
						}
					}
					//  F5..FF
				}
				//  80..C1
				break;
			}
			++begin;
		}
		str->clear();
		return false;
	}

	template <typename ForwardIterator>
	bool parse_value_(ForwardIterator &begin, const ForwardIterator end)
	{
		if (begin != end)
		{
			if (*begin == 0x5B)	//  '['
			{
				array_ = new array_type();
				begin = skip_spaces_(++begin, end);

				for (; begin != end;)
				{
					if (*begin == 0x5D)	//  ']'
					{
						begin = skip_spaces_(++begin, end);
						type_ = array;
						return true;
					}

					array_->resize(array_->size() + 1);
					if (!array_->back().parse_value_(begin, end))
						break;

					array_->back().type_ |= parsing;

					if (begin != end && *begin == 0x2C)	//  ','
						begin = skip_spaces_(++begin, end);
				}
				remove_parsingflag();
				delete array_;
				return false;
			}
			else if (*begin == 0x7B)	//  '{'
			{
				string_type key;

				object_ = new object_type();
				begin = skip_spaces_(++begin, end);

				for (; begin != end;)
				{
					if (*begin == 0x7D)	//  '}'
					{
						begin = skip_spaces_(++begin, end);
						type_ = object;
						return true;
					}

					if (*begin == 0x22)	//  '"', key.
					{
						key.clear();
						parse_string_(&key, ++begin, end, utftype());

						if (key.size() == 0 || begin == end || *begin != 0x3A)	//  ':'
							break;

						begin = skip_spaces_(++begin, end);

						typename object_core::iterator it = object_->objmap.find(key);
						jsonnode *newnode;

						if (it != object_->objmap.end())
						{
							it->second.remove_parsingflag();
							it->second.destroy_node_();
							it->second.type_ = unassigned;
							newnode = &it->second;
						}
						else
						{
							const safe_keysize keylen0(static_cast<keysize_type>(key.size()));
							object_->order.append(reinterpret_cast<const char_type *>(&keylen0.size), keysize_size);
							object_->order.append(key);
							newnode = &object_->objmap[key];
						}

						if (!newnode->parse_value_(begin, end))
							break;

						newnode->type_ |= parsing;

						if (begin != end && *begin == 0x2C)	//  ','
							begin = skip_spaces_(++begin, end);
					}
					//  else if (*begin == ',')
					else
						break;
				}
				remove_parsingflag();
				delete object_;
				return false;
			}
			else if (*begin == 0x22)	//  '"'
			{
				string_ = new string_type();

				if (parse_string_(string_, ++begin, end, utftype()))
				{
					type_ = string;
					return true;
				}
				delete string_;
			}
			else if (*begin == 0x74)	//  't'
			{
				if (++begin != end && *begin == 0x72)	//  'r'
					if (++begin != end && *begin == 0x75)	//  'u'
						if (++begin != end && *begin == 0x65)	//  'e'
						{
							++begin;
							type_ = boolean;
							boolvalue_ = true;
							return true;
						}
			}
			else if (*begin == 0x66)	//  'f'
			{
				if (++begin != end && *begin == 0x61)	//  'a'
					if (++begin != end && *begin == 0x6C)	//  'l'
						if (++begin != end && *begin == 0x73)	//  's'
							if (++begin != end && *begin == 0x65)	//  'e'
							{
								++begin;
								type_ = boolean;
								boolvalue_ = false;
								return true;
							}
			}
			else if (*begin == 0x6E)	//  'n'
			{
				if (++begin != end && *begin == 0x75)	//  'u'
					if (++begin != end && *begin == 0x6C)	//  'l'
						if (++begin != end && *begin == 0x6C)	//  'l'
						{
							++begin;
							type_ = null;
							return true;
						}
			}
			else
			{
				ForwardIterator orgbegin = begin;
				std::basic_string<char, std::char_traits<char>, alloc<char> > cdigits;

				if (*begin == 0x2D)	//  '-'
				{
					cdigits.append(1, '-');
					++begin;
				}

				if (begin != end)
				{
					if (*begin == 0x30)	//  '0'
					{
						cdigits.append(1, '0');
						++begin;
					}
					else if (*begin >= 0x31 && *begin <= 0x39)	//  '1', '9'.
					{
						do
						{
							cdigits.append(1, static_cast<char>(*begin - 0x30 + '0'));
						}
						while (++begin != end && *begin >= 0x30 && *begin <= 0x39);	//  '0', '9'.
					}
					else
						goto FAILED;

					if (begin != end && *begin == 0x2E)	//  '.'
					{
						cdigits.append(1, '.');
						if (++begin == end || *begin < 0x30 || *begin > 0x39)
							goto FAILED;

						do
						{
							cdigits.append(1, static_cast<char>(*begin - 0x30 + '0'));
						}
						while (++begin != end && *begin >= 0x30 && *begin <= 0x39);	//  '0', '9'.
					}

					if (begin != end && (*begin | 0x20) == 0x65)	//  'E' or 'e'.
					{
						cdigits.append(1, 'E');
						if (++begin == end)
							goto FAILED;

						if (*begin == 0x2B || *begin == 0x2D)	//  '+', '-'.
						{
							cdigits.append(1, *begin == 0x2B ? '+' : '-');
							++begin;
						}

						if (begin == end || *begin < 0x30 || *begin > 0x39)
							goto FAILED;

						do
						{
							cdigits.append(1, static_cast<char>(*begin - 0x30 + '0'));
						}
						while (++begin != end && *begin >= 0x30 && *begin <= 0x39);	//  '0', '9'.
					}

					type_ = number;
					const safe_double dbl(std::strtod(cdigits.c_str(), NULL));
					const char_type *const dblptr = reinterpret_cast<const char_type *>(&dbl.dbl);
					string_ = new string_type(dblptr, dblptr + double_size);
					string_->append(orgbegin, begin);
					return true;
				}
			}
		}
		FAILED:
		return false;
	}

	void destroy_node_()
	{
		if (type_ == array && array_)
		{
//			for (typename array_type::size_type i = 0; i < array_->size(); ++i)
//				(*array_)[i].destroy_node_();
			delete array_;
		}
		else if (type_ == object && object_)
		{
//			for (typename object_type::iterator it = object_->begin(); it != object_->end(); ++it)
//				it->second.destroy_node_();
			delete object_;
		}
		else if ((type_ == string || type_ == number) && string_)
			delete string_;
//		else	//  null, boolean.

//		value_ = NULL;
//		type_ = unassigned;
	}

	void remove_parsingflag()
	{
		type_ &= ~parsing;

		if (type_ == array)
		{
			for (typename array_type::size_type i = 0; i < array_->size(); ++i)
				(*array_)[i].remove_parsingflag();
		}
		else if (type_ == object)
		{
			for (typename object_core::iterator it = object_->objmap.begin(); it != object_->objmap.end(); ++it)
				it->second.remove_parsingflag();
		}
	}

	template <typename StringT>
	void value_to_string_(StringT &out) const
	{
		typedef typename StringT::value_type dchar_type;
		static const dchar_type quot = 0x22;	//  '"'
		static const dchar_type quotcolon[] = { 0x22, 0x3A, 0 };	//  "\":"
		static const dchar_type ltrue[] = { 0x74, 0x72, 0x75, 0x65, 0 };	//  "true"
		static const dchar_type lfalse[] = { 0x66, 0x61, 0x6C, 0x73, 0x65, 0 };	//  "false"
		static const dchar_type lnull[] = { 0x6E, 0x75, 0x6C, 0x6C, 0 };	//  "null"

		if (type_ == number)
		{
			const strsize_type base = out.size();

			out.resize(out.size() + string_->size() - double_size);

			typename StringT::iterator dst = out.begin() + base;
			typename string_type::const_iterator src = string_->begin() + double_size;

			for (; src != string_->end(); ++dst, ++src)
				*dst = static_cast<dchar_type>(*src);	//  ASCII only.
		}
		else if (type_ == string)
		{
			out.append(1, quot);
			escape_(out, *string_, utftype());
			out.append(1, quot);
		}
		else if (type_ == boolean)
			out.append(boolvalue_ ? ltrue : lfalse);
		else if (type_ == array)
		{
			out.append(1, 0x5B);	//  '['

			const strsize_type base = out.size();

			if (array_)
			{
				for (typename array_type::size_type i = 0; i < array_->size(); ++i)
				{
					(*array_)[i].value_to_string_(out);
					out.append(1, 0x2C);	//  ','
				}
			}

			if (out.size() != base)
				out[out.size() - 1] = 0x5D;	//  ']'
			else
				out.append(1, 0x5D);	//  ']'
		}
		else if (type_ == object)
		{
			out.append(1, 0x7B);	//  '{'

			const strsize_type base = out.size();

			if (object_)
			{
#if 0
				for (typename object_core::const_iterator it = object_->objmap.begin(); it != object_->objmap.end(); ++it)
				{
					out.append(1, quot);
					escape_(out, it->first, utftype());
					out.append(quotcolon);
					it->second.value_to_string_(out);
					out.append(1, 0x2C);	//  ','
				}
#else
				for (strsize_type pos = 0; (pos + keysize_size) < object_->order.size();)
				{
					const keysize_type len = *reinterpret_cast<const keysize_type *>(&object_->order[pos]);

					pos += keysize_size;
					const string_type key(object_->order.substr(pos, len));
					pos += len;
					typename object_core::const_iterator it = object_->objmap.find(key);

					if (it != object_->objmap.end())
					{
						out.append(1, quot);
						escape_(out, it->first, utftype());
						out.append(quotcolon);
						it->second.value_to_string_(out);
						out.append(1, 0x2C);	//  ','
					}
				}
#endif
			}

			if (out.size() != base)
				out[out.size() - 1] = 0x7D;	//  '}'
			else
				out.append(1, 0x7D);	//  '}'
		}
		else
			out.append(lnull);
	}

	void remove_orderkey_(const string_type &key)
	{
		for (strsize_type pos = 0; (pos + keysize_size) < object_->order.size();)
		{
			const strsize_type keylen = static_cast<strsize_type>(*reinterpret_cast<const keysize_type *>(&object_->order[pos]));
			const strsize_type keybase = pos + keysize_size;

			if (keylen == key.size())
			{
				for (strsize_type i = 0;; ++i)
				{
					if (i == keylen)
					{
						object_->order.erase(pos, keysize_size + keylen);
						return;
					}
					if (object_->order[keybase + i] != key[i])
						break;
				}
			}
			pos = keybase + keylen;
		}
	}

	template <typename ForwardIterator>
	unsigned long parse_4hexdigits_(ForwardIterator &begin, const ForwardIterator end) const
	{
		unsigned long ucp = 0ul;

		for (strsize_type i = 0; i < 4; ++i)
		{
			if (++begin != end)
			{
				int b = *begin;

				if (b >= 0x30 && b <= 0x39)
					ucp = (ucp << 4) | (b - 0x30);
				else
				{
					b |= 0x20;
					if (b >= 0x61 && b <= 0x66)
						ucp = (ucp << 4) | (b - 0x61 + 10);
					else
						return 0x10000ul;
				}
			}
			else
				break;
		}
		return ucp;
	}

	//  UTF-8 -> UTF-8.

	template <typename StringT, typename UTFtype>
	void escape_(StringT &d, const string_type &s, const UTFtype) const
	{
		typedef typename StringT::value_type dchar_type;
		typedef typename StringT::size_type dsize_type;

		const dsize_type base = d.size();

		d.append(s.begin(), s.end());

		for (dsize_type i = base; i < d.size(); ++i)
		{
			unsigned long ch = d[i];

			//  unescaped = %x20-21 / %x23-5B / %x5D-10FFFF
			if (ch < 0x20)
			{
				d.insert(i, 5, 0);
				d[i++] = static_cast<dchar_type>(0x5c);
				d[i++] = static_cast<dchar_type>(0x75);
				d[i++] = static_cast<dchar_type>(0x30);
				d[i++] = static_cast<dchar_type>(0x30);
				d[i++] = static_cast<dchar_type>((ch >> 4) + 0x30);
				ch &= 0xf;
				d[i] = static_cast<dchar_type>(ch + (ch < 10 ? 0x30 : 0x41 - 10));
			}
			else if (ch == 0x22 || ch == 0x5c)
			{
				d.insert(i++, 1, 0x5c);
			}
		}
	}

	//  UTF-16 -> UTF-8.

	template <typename StringT>
	void escape_(StringT &d, const string_type &s, const utf16tag) const
	{
		typedef typename StringT::value_type dchar_type;
		typedef typename StringT::size_type dsize_type;
		unsigned long prev = 0ul;

		d.reserve(d.size() + s.size());

		for (strsize_type i = 0; i < s.size(); ++i)
		{
			unsigned long ch = s[i] & 0xffff;

			//  unescaped = %x20-21 / %x23-5B / %x5D-10FFFF
			if (ch < 0x20)
			{
				dsize_type j = d.size();

				d.resize(j + 6);
				d[j++] = static_cast<dchar_type>(0x5c);
				d[j++] = static_cast<dchar_type>(0x75);
				d[j++] = static_cast<dchar_type>(0x30);
				d[j++] = static_cast<dchar_type>(0x30);
				d[j++] = static_cast<dchar_type>((ch >> 4) + 0x30);
				ch &= 0xf;
				d[j] = static_cast<dchar_type>(ch + (ch < 10 ? 0x30 : 0x41 - 10));
			}
			else if (ch == 0x22 || ch == 0x5c)
			{
				d.resize(d.size() + 2, 0x5c);
				d[d.size() - 1] = static_cast<dchar_type>(ch);
			}
			else if (ch >= 0x80)
			{
				if (ch >= 0x800)
				{
					if (prev && ((ch - 0xdc00) < 0x400))	//  DC00..DFFF.
					{
						dsize_type j = d.size();

						ch = (((prev & 0x3ff) << 10) | (ch & 0x3ff)) + 0x10000;

						d.resize(j + 1, static_cast<dchar_type>(0x80 | (ch & 0x3f)));
						d[j - 3] = static_cast<dchar_type>(0xf0 | (ch >> 18));
						d[j - 2] = static_cast<dchar_type>(0x80 | ((ch >> 12) & 0x3f));
						d[j - 1] = static_cast<dchar_type>(0x80 | ((ch >> 6) & 0x3f));
					}
					else
					{
						dsize_type j = d.size();

						d.resize(j + 3, static_cast<dchar_type>(0x80 | (ch & 0x3f)));
						d[j++] = static_cast<dchar_type>(0xe0 | (ch >> 12));
						d[j++] = static_cast<dchar_type>(0x80 | ((ch >> 6) & 0x3f));

						prev = ((ch - 0xd800) < 0x400) ? ch : 0ul;	//  D800..DBFF.
						continue;
					}
				}
				else	//  80..7FF.
				{
					dsize_type j = d.size();

					d.resize(j + 2, static_cast<dchar_type>(0x80 | (ch & 0x3f)));
					d[j] = static_cast<dchar_type>(0xc0 | (ch >> 6));
				}
			}
			else
				d.push_back(static_cast<dchar_type>(ch));

			prev = 0ul;
		}
	}

private:

	union
	{
		void *value_;
		string_type *string_;
		array_type *array_;
		object_type *object_;
		bool boolvalue_;
	};
	union
	{
//		value_type type_;
		int type_;
		void *padding_;
	};

	static jsonnode fallback_node_;
};
//  Thanks to ODR, only one static object exists per type of charT,
//  as fallback_node_.fallback_node_ is also jsonnode<charT>.
#if defined(__cplusplus) && (__cplusplus >= 201103L)
template <typename charT, typename utftype,
	template <typename V1, typename... V2> class vector,
	template <typename M1, typename M2, typename... M3> class map,
#else
template <typename charT, typename utftype,
	template <typename V1, typename V2> class vector,
	template <typename M1, typename M2, typename M3, typename M4> class map,
#endif
	template <typename A1> class alloc>
jsonnode<charT, utftype, vector, map, alloc> jsonnode<charT, utftype, vector, map, alloc>::fallback_node_(jsonnode<charT, utftype, vector, map, alloc>::fallback);
//  jsonnode

	}	//  namespace json_internal_

typedef json_internal_::jsonnode<char, json_internal_::utf8tag> json;
typedef json_internal_::jsonnode<char, json_internal_::utf8tag> u8cjson;

#if defined(WCHAR_MAX) && (WCHAR_MAX >= 0xffff)
#if (WCHAR_MAX >= 0x10ffff)
#else
typedef json_internal_::jsonnode<wchar_t, json_internal_::utf16tag> wjson;
typedef json_internal_::jsonnode<wchar_t, json_internal_::utf16tag> u16wjson;
#endif
#endif

#if defined(__cpp_unicode_characters)
typedef json_internal_::jsonnode<char16_t, json_internal_::utf16tag> u16json;
#endif

#if defined(__cpp_char8_t)
typedef json_internal_::jsonnode<char8_t, json_internal_::utf8tag> u8json;
#endif

}	//  namespace NAMESPACE_CTLJSON
#endif	//  CTL_JSON_TEMPLATE_LIBRARY
